/*
 * ocamlsdl - Objective Caml interface for the SDL library
 * Copyright (C) 1999 Jean-Christophe FILLIATRE
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2, as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU Library General Public License version 2 for more details
 * (enclosed in the file LGPL).
 */

/* $Id: draw_stub.c,v 1.3 2006-09-20 17:34:00 filliatr Exp $ */

#include <stdio.h>
#include <SDL.h>
#include <caml/mlvalues.h>
#include <caml/memory.h>
#include <caml/fail.h>
#include <caml/callback.h>

/* the following code is borrowed from stars.c by Nathan Strong */

value sdldraw_put_pixel(value s, value vx, value vy, value vc) {
  Uint16 x = Int_val(vx);
  Uint16 y = Int_val(vy);
  Uint32 pixel = Int_val(vc);
  SDL_Surface* dst = (SDL_Surface*) s;
  Uint8 *pix;
  int shift;

  if (SDL_LockSurface(dst) == 0) {
    switch (dst->format->BytesPerPixel) {
    case 1:
      *((Uint8 *)dst->pixels + y * dst->pitch + x) = pixel;
      break;
    case 2:
      *((Uint16 *)dst->pixels + y * dst->pitch/2 + x) = pixel;
      break;
    case 3:
      /* Slow, but endian correct */
      pix = (Uint8 *)dst->pixels + y * dst->pitch + x*3;
      shift = dst->format->Rshift;
      *(pix+shift/8) = pixel>>shift;
      shift = dst->format->Gshift;
      *(pix+shift/8) = pixel>>shift;
      shift = dst->format->Bshift;
      *(pix+shift/8) = pixel>>shift;
      break;
    case 4:
      *((Uint32 *)dst->pixels + y * dst->pitch/4 + x) = pixel;
      break;
    };
    SDL_UnlockSurface(dst);
  };
  return Val_unit;
}

value sdldraw_get_pixel(value s, value vx, value vy) {
  Uint16 x = Int_val(vx);
  Uint16 y = Int_val(vy);
  Uint32 pixel;
  SDL_Surface* dst = (SDL_Surface*) s;
  Uint8 *pix;
  int shift;

  if (SDL_LockSurface(dst) == 0) {
    switch (dst->format->BytesPerPixel) {
    case 1:
      pixel = *((Uint8 *)dst->pixels + y * dst->pitch + x);
      break;
    case 2:
      pixel = *((Uint16 *)dst->pixels + y * dst->pitch/2 + x);
      break;
    case 3:
      /* Slow, but endian correct */
      pix = (Uint8 *)dst->pixels + y * dst->pitch + x*3;
      pixel = 0;
      shift = dst->format->Rshift;
      pixel |= (*(pix+shift/8)) << shift;
      shift = dst->format->Gshift;
      pixel |= (*(pix+shift/8)) << shift;
      shift = dst->format->Bshift;
      pixel |= (*(pix+shift/8)) << shift;
      break;
    case 4:
      pixel = *((Uint32 *)dst->pixels + y * dst->pitch/4 + x);
      break;
    };
    SDL_UnlockSurface(dst);
  };

  return Val_int(pixel);
}
