
open Printf

open Sdl
open Video
open Event

open GlDraw
open GlPix
open GlTex

(* floats for x rotation, y rotation, z rotation *)
let xrot = ref 0.0
let yrot = ref 0.0
let zrot = ref 0.0
let angle = 0.5
let obs = ref (-3.0)

let (pix1 : ([< `rgb ], [< `ubyte ]) GlPix.t) = 
  let s1 = GL.load_bmp "earth.bmp" in
  let raw1 = Raw.of_string (string_of_pixels s1) `ubyte in
  of_raw raw1 `rgb (surface_width s1) (surface_height s1)

let tex1 = GlTex.gen_texture ()

(* Load Bitmaps And Convert To Textures*)
let load_textures () =
  (* Create Texture	*)
  GlTex.bind_texture `texture_2d tex1;   (* 2d texture (x and y size)*)
  GlTex.parameter `texture_2d (`mag_filter `linear); (* scale linearly when image bigger than texture*)
  GlTex.parameter `texture_2d (`min_filter `linear); (* scale linearly when image smalled than texture*)

    (* 2d texture, level of detail 0 (normal), 3 components (red, green, blue), x size from image, y size from image, *)
    (* border 0 (normal), rgb color data, unsigned byte data, and finally the data itself.*)
  GlTex.image2d ~proxy:false ~level:0 ~internal:3 ~border:false pix1

(* A general OpenGL initialization function.  Sets all of the initial parameters. *)
let init_gl width height = 	        (* We call this right after our OpenGL window is created.*)
  GlDraw.viewport 0 0 width height;
  load_textures();				(* Load The Texture(s) *)
  Gl.enable (`texture_2d);			(* Enable Texture Mapping*)
  GlClear.color (0.0, 0.0, 0.0);     (* Clear The Background Color To Blue *)
  GlClear.depth (1.0);				(* Enables Clearing Of The Depth Buffer*)
  GlFunc.depth_func (`less);			(* The Type Of Depth Test To Do*)
  Gl.enable (`depth_test);			(* Enables Depth Testing*)
(*  GlDraw.shade_model (`smooth);			(* Enables Smooth Color Shading*) *)
  
  GlMat.mode (`projection);
  GlMat.load_identity ();				(* Reset The Projection Matrix*)
  
  GluMat.perspective ~fovy:45.0 ~aspect:(float width /. float height) 
    ~z:(0.1,100.0);	(* Calculate The Aspect Ratio Of The Window*)
  
  GlMat.mode (`modelview)

(* The main drawing function. *)
let draw_gl_scene () =
  GlClear.clear [`color ; `depth];		(* Clear The Screen And The Depth Buffer*)
  GlMat.load_identity ();				(* Reset The View*)
  
  GlMat.translate3 (0.0,0.0,!obs);     (* move into the screen.*)
  
  GlMat.rotate3 !xrot (1.0,0.0,0.0);		(* Rotate On The X Axis*)
  GlMat.rotate3 !yrot (0.0,1.0,0.0);		(* Rotate On The Y Axis*)
  GlMat.rotate3 !zrot (0.0,0.0,1.0);		(* Rotate On The Z Axis*)

  GlTex.bind_texture `texture_2d tex1;   (* choose the texture to use.*)

  let q = GluQuadric.create () in
  GluQuadric.normals q `flat;
  GluQuadric.texture q true;
  GluQuadric.sphere ~radius:1.0 ~slices:32 ~stacks:32 ~quad:q ();
  
  GlDraw.ends ();
  (* swap buffers to display, since we're double buffered.*)
  Sdl.GL.swap_buffers ()

let main () =
  let done_ = ref false in
  (* Initialize SDL for video output *)
  Sdl.init [VIDEO];
  (* Create a 640x480 OpenGL screen *)
  let screen = set_video_mode 640 480 0 [OPENGL] in
  (* Loop, drawing and checking events *)
  init_gl 640 480;
  while not !done_ do
    draw_gl_scene ();
    (* This could go in a separate function *)
xrot := !xrot +. angle;
(*
    match poll_event () with
      | Some Quit -> done_ := true
      | Some (KeyDown k) ->
	  (match k.key_sym.key with
	     | 27 -> done_ := true
	     | 273 (* up *) -> xrot := !xrot +. angle
	     | 274 (* down *) -> xrot := !xrot -. angle
	     | 122 (* a *) -> yrot := !yrot -. angle
	     | 97 (* z *) -> yrot := !yrot +. angle
	     | 276 (* left *) -> zrot := !zrot +. angle
	     | 275 (* right *) -> zrot := !zrot -. angle
	     | 112 (* p *) -> obs := !obs +. 1.
	     | 109 (* m *) -> obs := !obs -. 1.
	     | n -> eprintf "key = %d\n" n; flush stderr)
      | _ -> ()
*)
  done

let _ = try main (); Sdl.quit () with e -> Sdl.quit (); raise e
